import json
import logging

from django.core.serializers.json import DjangoJSONEncoder
from fernet_fields import EncryptedTextField

log = logging.getLogger(__name__)


class EncryptedJSONField(EncryptedTextField):
    """JSON dictionary that is encrypted at rest as a text object - based off of the django-fernet-fields EncryptedTextField"""

    def get_prep_value(self, value):
        ret_val = "{}"
        if value:
            ret_val = json.dumps(value, separators=(",", ":"), cls=DjangoJSONEncoder)
        return ret_val

    def to_python(self, value):
        ret_val = {}
        if isinstance(value, dict):
            ret_val = value
        elif isinstance(value, str):
            ret_val = json.loads(value)
        return ret_val



import logging
from io import BytesIO

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from django.core.files.images import ImageFile
from django.db.models.fields.files import FieldFile, FileField, ImageField
from django.urls import reverse

from ..conf import settings
from .dynamic_storage import (DynamicStorageFieldFile, DynamicStorageFileField,
                              DynamicStorageImageField,
                              DynamicStorageImageFieldFile, FileFieldMixin)

log = logging.getLogger(__name__)


class EncryptedFile(BytesIO):
    def __init__(self, content, fernet):
        if content:
            BytesIO.__init__(self, fernet.encrypt(content.read()))
        else:
            BytesIO.__init__(self, None)


class DecryptedFile(BytesIO):
    def __init__(self, content, fernet):
        if content:
            BytesIO.__init__(self, fernet.decrypt(content))
        else:
            BytesIO.__init__(self, None)


class EncryptedFieldFile(DynamicStorageFieldFile):
    def __init__(self, instance, field, name, **kwargs):
        self.fernet = getattr(field, "fernet", None)
        super().__init__(instance, field, name)
        if callable(self.fernet):
            self.fernet = self.fernet(instance) if self.fernet and callable(self.fernet) else self.fernet
        if not self.fernet:
            raise RuntimeError("Encryption Fernet details not provided for this instance of a EncryptedFieldFile")

    @property
    def fernet(self):
        return getattr(self, "_fernet", None)

    @fernet.setter
    def fernet(self, val):
        self._fernet = val

    def _get_url(self):
        if settings.DS_DOWNLOAD_URL_NAME:
            url_args = []
            for k in settings.DS_DOWNLOAD_URL_ARG_NAMES:
                val = str(getattr(self.instance, k, None))
                if val:
                    url_args.append(val)
            return reverse(settings.DS_DOWNLOAD_URL_NAME, args=url_args)
        return None

    def get_decrypted(self):
        with self.open("rb") as f:
            return DecryptedFile(f, self.fernet)

    url = property(_get_url)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        if self.fernet:
            kwargs["fernet"] = self.fernet
        return name, path, args, kwargs

    def save(self, name, content, save=True):
        return DynamicStorageFieldFile.save(self, name, EncryptedFile(content, self.fernet), save=save)

    save.alters_data = True


class EncryptedFileField(DynamicStorageFileField):
    attr_class = EncryptedFieldFile

    def __init__(self, *args, **kwargs):
        self.fernet = kwargs.pop("fernet", None)
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        if self.fernet:
            kwargs["fernet"] = self.fernet
        return name, path, args, kwargs


class EncryptedImageFieldFile(ImageFile, EncryptedFieldFile):
    def delete(self, save=True):
        if hasattr(self, "_dimensions_cache"):
            del self._dimensions_cache
        super().delete(save)

    def save(self, name, content, save=True):
        return DynamicStorageImageFieldFile.save(self, name, EncryptedFile(content, self.fernet), save=save)


class EncryptedImageField(DynamicStorageImageField):
    attr_class = EncryptedImageFieldFile

    def __init__(self, *args, **kwargs):
        fernet = kwargs.pop("fernet", None)
        super().__init__(*args, **kwargs)
        self.fernet = fernet



# Generated by Django 3.1.2 on 2020-10-16 17:48

import uuid

import django.db.models.deletion
from django.db import migrations, models

import dynamic_storages.fields.dynamic_storage
import dynamic_storages.fields.encrypted_content
import dynamic_storages.fields.encrypted_json
import dynamic_storages.tests.models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="TestStorageTarget",
            fields=[
                ("id", models.UUIDField(default=uuid.uuid4, help_text="UUID identifying this objects", primary_key=True, serialize=False)),
                ("name", models.CharField(db_index=True, help_text="Name of this object", max_length=150)),
                ("description", models.TextField(blank=True, help_text="Description of this object", null=True)),
                ("last_checked", models.DateTimeField(blank=True, editable=False, help_text="Timestamp identifying when this storage provider was last checked", null=True)),
                (
                    "last_status",
                    models.CharField(
                        choices=[["v", "Valid"], ["e", "Error"], ["u", "Unknown"]],
                        default="u",
                        editable=False,
                        help_text="Flag indiciating what the last status check result was",
                        max_length=1,
                    ),
                ),
                ("status_detail", models.TextField(blank=True, editable=False, help_text="Status details from last status check", null=True)),
                (
                    "provider",
                    models.CharField(
                        choices=[
                            ("libcloud", "Apache LibCloud"),
                            ("azure", "Azure Blob Storage"),
                            ("dropbox", "Dropbox"),
                            ("ftp", "FTP"),
                            ("gcloud", "Google Cloud Storage"),
                            ("s3boto3", "S3/Boto3"),
                            ("do", "Digital Ocean (boto3)"),
                            ("sftp", "SFTP"),
                        ],
                        default="gcloud",
                        help_text="Specific storage provider this target utilizes - generally an object storage solution of some sort",
                        max_length=8,
                    ),
                ),
                (
                    "config",
                    dynamic_storages.fields.encrypted_json.EncryptedJSONField(
                        blank=True, default=dict, help_text="Key/value pairs to pass to the storage provider when initializing the storage backend", null=True
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True, db_index=True, help_text="Timestamp indicating when this object was created")),
                ("modified", models.DateTimeField(auto_now=True, db_index=True, help_text="Timestamp indicating when this object was last updated")),
            ],
            options={
                "verbose_name": "Storage Target",
                "verbose_name_plural": "Storage Targets",
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="TestImageStorageModel",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("image", dynamic_storages.fields.dynamic_storage.DynamicStorageImageField(upload_to=dynamic_storages.tests.models.upload_to)),
                ("storage_target", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="tests.teststoragetarget")),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="TestFileStorageModel",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("file", dynamic_storages.fields.dynamic_storage.DynamicStorageFileField(upload_to=dynamic_storages.tests.models.upload_to)),
                ("storage_target", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="tests.teststoragetarget")),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="TestEncryptedImageFieldModel",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("image", dynamic_storages.fields.encrypted_content.EncryptedImageField(upload_to=dynamic_storages.tests.models.upload_to)),
                ("key", models.CharField(default=dynamic_storages.tests.models.gen_key, editable=False, max_length=60)),
                ("storage_target", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="tests.teststoragetarget")),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="TestEncryptedFileFieldModel",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "file",
                    dynamic_storages.fields.encrypted_content.EncryptedFileField(
                        fernet=dynamic_storages.tests.models.get_fernet, upload_to=dynamic_storages.tests.models.upload_to
                    ),
                ),
                ("key", models.CharField(default=dynamic_storages.tests.models.gen_key, editable=False, max_length=60)),
                ("storage_target", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="tests.teststoragetarget")),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



import os

from appconf import AppConf
from django.conf import settings


class DynamicStoragesAppConf(AppConf):
    DOWNLOAD_REQUIRE_LOGIN = bool(os.getenv("DS_DOWNLOAD_REQUIRE_LOGIN", True))
    DOWNLOAD_URL_ARG_NAMES = os.getenv("DS_DOWNLOAD_URL_ARG_NAMES", "pk").split(",")
    DOWNLOAD_URL_KWARG_NAMES = os.getenv("DS_DOWNLOAD_URL_KWARG_NAMES", "pk").split(",")
    DOWNLOAD_INCLUDE_FILE_EXT = bool(os.getenv("DS_DOWNLOAD_INCLUDE_FILE_EXT", True))
    DOWNLOAD_URL_NAME = os.getenv("DOWNLOAD_URL_NAME", "")  # encrypted_file_download

    class Meta:
        prefix = "DS"
